@@ -81,34 +81,56 @@ class LDAPBackend(object):
     supports_object_permissions = True
     supports_inactive_user = False
 
-    ldap = None # The cached ldap module (or mock object)
+    _settings = None
+    _ldap = None # The cached ldap module (or mock object)
 
     # This is prepended to our internal setting names to produce the names we
     # expect in Django's settings file. Subclasses can change this in order to
     # support multiple collections of settings.
     settings_prefix = 'AUTH_LDAP_'
 
-    def __init__(self):
-        self.settings = LDAPSettings(self.settings_prefix)
-        self.ldap = self.ldap_module()
-
-    def ldap_module(self):
+    def __getstate__(self):
         """
-        Requests the ldap module from _LDAPConfig. Under a test harness, this
-        will be a mock object.
+        Exclude certain cached properties from pickling.
         """
-        from django.conf import settings
+        state = filter(
+            lambda (k, v): k not in ['_settings', '_ldap'],
+            self.__dict__.iteritems()
+        )
 
-        options = getattr(settings, 'AUTH_LDAP_GLOBAL_OPTIONS', None)
+        return dict(state)
 
-        return _LDAPConfig.get_ldap(options)
+    def _get_settings(self):
+        if self._settings is None:
+            self._settings = LDAPSettings(self.settings_prefix)
 
+        return self._settings
+
+    def _set_settings(self, settings):
+        self._settings = settings
+
+    settings = property(_get_settings, _set_settings)
+
+    def _get_ldap(self):
+        if self._ldap is None:
+            from django.conf import settings
+
+            options = getattr(settings, 'AUTH_LDAP_GLOBAL_OPTIONS', None)
+
+            self._ldap = _LDAPConfig.get_ldap(options)
+
+        return self._ldap
+    ldap = property(_get_ldap)
 
     #
     # The Django auth backend API
     #
 
     def authenticate(self, username, password):
+        if len(password) == 0 and not self.settings.PERMIT_EMPTY_PASSWORD:
+            logger.debug('Rejecting empty password for %s' % username)
+            return None
+
         ldap_user = _LDAPUser(self, username=username.strip())
         user = ldap_user.authenticate(password)
 
@@ -191,6 +213,15 @@ class _LDAPUser(object):
     class AuthenticationFailed(Exception):
         pass
 
+    # Defaults
+    _user = None
+    _user_dn = None
+    _user_attrs = None
+    _groups = None
+    _group_permissions = None
+    _connection = None
+    _connection_bound = False
+
     #
     # Initialization
     #
@@ -202,16 +233,7 @@ class _LDAPUser(object):
         ignored.
         """
         self.backend = backend
-        self.ldap = backend.ldap
-        self.settings = backend.settings
         self._username = username
-        self._user_dn = None
-        self._user_attrs = None
-        self._user = None
-        self._groups = None
-        self._group_permissions = None
-        self._connection = None
-        self._connection_bound = False  # True if we're bound as AUTH_LDAP_BIND_*
 
         if user is not None:
             self._set_authenticated_user(user)
@@ -222,7 +244,6 @@ class _LDAPUser(object):
     def __deepcopy__(self, memo):
         obj = object.__new__(self.__class__)
         obj.backend = self.backend
-        obj.ldap = self.ldap
         obj._user = copy.deepcopy(self._user, memo)
 
         # This is all just cached immutable data. There's no point copying it.
@@ -238,6 +259,18 @@ class _LDAPUser(object):
 
         return obj
 
+    def __getstate__(self):
+        """
+        Most of our properties are cached from the LDAP server. We only want to
+        pickle a few crucial things.
+        """
+        state = filter(
+            lambda (k, v): k in ['backend', '_username', '_user'],
+            self.__dict__.iteritems()
+        )
+
+        return dict(state)
+
     def _set_authenticated_user(self, user):
         self._user = user
         self._username = self.backend.django_to_ldap_username(user.username)
@@ -245,6 +278,14 @@ class _LDAPUser(object):
         user.ldap_user = self
         user.ldap_username = self._username
 
+    def _get_ldap(self):
+        return self.backend.ldap
+    ldap = property(_get_ldap)
+
+    def _get_settings(self):
+        return self.backend.settings
+    settings = property(_get_settings)
+
     #
     # Entry points
     #
@@ -267,10 +308,9 @@ class _LDAPUser(object):
         except self.ldap.LDAPError, e:
             logger.warning(u"Caught LDAPError while authenticating %s: %s",
                 self._username, pprint.pformat(e))
-        except Exception, e:
-            logger.error(u"Caught Exception while authenticating %s: %s",
-                self._username, pprint.pformat(e))
-            logger.error(''.join(traceback.format_tb(sys.exc_info()[2])))
+        except Exception:
+            logger.exception(u"Caught Exception while authenticating %s",
+                self._username)
             raise
 
         return user
@@ -647,7 +687,6 @@ class _LDAPUser(object):
         return self._connection
 
 
-
 class _LDAPUserGroups(object):
     """
     Represents the set of groups that a user belongs to.
@@ -778,6 +817,7 @@ class LDAPSettings(object):
         'GROUP_SEARCH': None,
         'GROUP_TYPE': None,
         'MIRROR_GROUPS': False,
+        'PERMIT_EMPTY_PASSWORD': False,
         'PROFILE_ATTR_MAP': {},
         'PROFILE_FLAGS_BY_GROUP': {},
         'REQUIRE_GROUP': None,
