@@ -29,9 +29,10 @@ try:
 except NameError:
     from sets import Set as set     # Python 2.3 fallback
 
-import sys
-import logging
 from collections import defaultdict
+import logging
+import pickle
+import sys
 
 from django.conf import settings
 import django.db.models.signals
@@ -82,8 +83,8 @@ class MockLDAP(object):
     ldap_methods_called() to get a record of all of the LDAP API calls that have
     been made, with or without arguments.
     """
-
-    class PresetReturnRequiredError(Exception): pass
+    class PresetReturnRequiredError(Exception):
+        pass
 
     SCOPE_BASE = 0
     SCOPE_ONELEVEL = 1
@@ -91,10 +92,17 @@ class MockLDAP(object):
 
     RES_SEARCH_RESULT = 101
 
-    class LDAPError(Exception): pass
-    class INVALID_CREDENTIALS(LDAPError): pass
-    class NO_SUCH_OBJECT(LDAPError): pass
-    class NO_SUCH_ATTRIBUTE(LDAPError): pass
+    class LDAPError(Exception):
+        pass
+
+    class INVALID_CREDENTIALS(LDAPError):
+        pass
+
+    class NO_SUCH_OBJECT(LDAPError):
+        pass
+
+    class NO_SUCH_ATTRIBUTE(LDAPError):
+        pass
 
     #
     # Submodules
@@ -113,7 +121,6 @@ class MockLDAP(object):
         class cidict(dict):
             pass
 
-
     def __init__(self, directory):
         """
         directory is a complex structure with the entire contents of the
@@ -208,9 +215,9 @@ class MockLDAP(object):
         self._record_call('search', {
             'base': base,
             'scope': scope,
-            'filterstr':filterstr,
-            'attrlist':attrlist,
-            'attrsonly':attrsonly
+            'filterstr': filterstr,
+            'attrlist': attrlist,
+            'attrsonly': attrsonly
         })
 
         value = self._get_return_value('search_s',
@@ -233,9 +240,9 @@ class MockLDAP(object):
         self._record_call('search_s', {
             'base': base,
             'scope': scope,
-            'filterstr':filterstr,
-            'attrlist':attrlist,
-            'attrsonly':attrsonly
+            'filterstr': filterstr,
+            'attrlist': attrlist,
+            'attrsonly': attrsonly
         })
 
         value = self._get_return_value('search_s',
@@ -342,7 +349,6 @@ class MockLDAP(object):
 
 
 class LDAPTest(TestCase):
-
     # Following are the objecgs in our mock LDAP directory
     alice = ("uid=alice,ou=people,o=test", {
         "uid": ["alice"],
@@ -434,7 +440,6 @@ class LDAPTest(TestCase):
         "member": ["cn=parent_gon,ou=groups,o=test"]
     })
 
-
     mock_ldap = MockLDAP({
         alice[0]: alice[1],
         bob[0]: bob[1],
@@ -451,8 +456,8 @@ class LDAPTest(TestCase):
         circular_gon[0]: circular_gon[1],
     })
 
-
     logging_configured = False
+
     def configure_logger(cls):
         if not cls.logging_configured:
             logger = logging.getLogger('django_auth_ldap')
@@ -468,19 +473,19 @@ class LDAPTest(TestCase):
             cls.logging_configured = True
     configure_logger = classmethod(configure_logger)
 
-
     def setUp(self):
         self.configure_logger()
-        self.mock_ldap.reset()
 
         self.ldap = _LDAPConfig.ldap = self.mock_ldap
-        self.backend = backend.LDAPBackend()
 
+        self.backend = backend.LDAPBackend()
+        self.backend.ldap # Force global configuration
+
+        self.mock_ldap.reset()
 
     def tearDown(self):
         pass
 
-
     def test_options(self):
         self._init_settings(
             USER_DN_TEMPLATE='uid=%(user)s,ou=people,o=test',
@@ -532,7 +537,6 @@ class LDAPTest(TestCase):
         self.assertEqual(user.username, 'alice')
         self.assertEqual(User.objects.count(), user_count + 1)
 
-
     def test_simple_bind_bad_user(self):
         self._init_settings(
             USER_DN_TEMPLATE='uid=%(user)s,ou=people,o=test'
@@ -576,8 +580,8 @@ class LDAPTest(TestCase):
         self._init_settings(
             USER_SEARCH=LDAPSearch(
                 "ou=people,o=test", self.mock_ldap.SCOPE_SUBTREE, '(uid=%(user)s)'
-                )
             )
+        )
         self.mock_ldap.set_return_value('search_s',
             ("ou=people,o=test", 2, "(uid=Alice)", None, 0), [self.alice])
         User.objects.create(username='alice')
@@ -592,6 +596,7 @@ class LDAPTest(TestCase):
         class MyBackend(backend.LDAPBackend):
             def ldap_to_django_username(self, username):
                 return 'ldap_%s' % username
+
             def django_to_ldap_username(self, username):
                 return username[5:]
 
@@ -616,8 +621,8 @@ class LDAPTest(TestCase):
         self._init_settings(
             USER_SEARCH=LDAPSearch(
                 "ou=people,o=test", self.mock_ldap.SCOPE_SUBTREE, '(uid=%(user)s)'
-                )
             )
+        )
         self.mock_ldap.set_return_value('search_s',
             ("ou=people,o=test", 2, "(uid=alice)", None, 0), [self.alice])
         user_count = User.objects.count()
@@ -633,8 +638,8 @@ class LDAPTest(TestCase):
         self._init_settings(
             USER_SEARCH=LDAPSearch(
                 "ou=people,o=test", self.mock_ldap.SCOPE_SUBTREE, '(cn=%(user)s)'
-                )
             )
+        )
         self.mock_ldap.set_return_value('search_s',
             ("ou=people,o=test", 2, "(cn=alice)", None, 0), [])
 
@@ -648,8 +653,8 @@ class LDAPTest(TestCase):
         self._init_settings(
             USER_SEARCH=LDAPSearch(
                 "ou=people,o=test", self.mock_ldap.SCOPE_SUBTREE, '(uid=*)'
-                )
             )
+        )
         self.mock_ldap.set_return_value('search_s',
             ("ou=people,o=test", 2, "(uid=*)", None, 0), [self.alice, self.bob])
 
@@ -663,8 +668,8 @@ class LDAPTest(TestCase):
         self._init_settings(
             USER_SEARCH=LDAPSearch(
                 "ou=people,o=test", self.mock_ldap.SCOPE_SUBTREE, '(uid=%(user)s)'
-                )
             )
+        )
         self.mock_ldap.set_return_value('search_s',
             ("ou=people,o=test", 2, "(uid=alice)", None, 0), [self.alice])
 
@@ -680,8 +685,8 @@ class LDAPTest(TestCase):
             BIND_PASSWORD='password',
             USER_SEARCH=LDAPSearch(
                 "ou=people,o=test", self.mock_ldap.SCOPE_SUBTREE, '(uid=%(user)s)'
-                )
             )
+        )
         self.mock_ldap.set_return_value('search_s',
             ("ou=people,o=test", 2, "(uid=alice)", None, 0), [self.alice])
 
@@ -700,8 +705,8 @@ class LDAPTest(TestCase):
             BIND_PASSWORD='bogus',
             USER_SEARCH=LDAPSearch(
                 "ou=people,o=test", self.mock_ldap.SCOPE_SUBTREE, '(uid=%(user)s)'
-                )
             )
+        )
 
         user = self.backend.authenticate(username='alice', password='password')
 
@@ -767,6 +772,7 @@ class LDAPTest(TestCase):
         self._init_settings(
             USER_DN_TEMPLATE='uid=%(user)s,ou=people,o=test'
         )
+
         def handle_populate_user(sender, **kwargs):
             self.assert_('user' in kwargs and 'ldap_user' in kwargs)
             kwargs['user'].populate_user_handled = True
@@ -998,7 +1004,6 @@ class LDAPTest(TestCase):
         self.assert_(alice.get_profile().is_special)
         self.assert_(not bob.get_profile().is_special)
 
-
     def test_dn_group_permissions(self):
         self._init_settings(
             USER_DN_TEMPLATE='uid=%(user)s,ou=people,o=test',
@@ -1271,8 +1276,8 @@ class LDAPTest(TestCase):
         self._init_settings(
             USER_SEARCH=LDAPSearch(
                 "ou=people,o=test", self.mock_ldap.SCOPE_SUBTREE, '(uid=%(user)s)'
-                )
             )
+        )
         self.mock_ldap.set_return_value('search_s',
             ("ou=people,o=test", 2, "(uid=alice)", None, 0), [self.alice, (None, '')])
 
@@ -1298,6 +1303,53 @@ class LDAPTest(TestCase):
             ['initialize', 'simple_bind_s', 'search', 'search', 'result',
                 'result', 'simple_bind_s'])
 
+    def test_deny_empty_password(self):
+        self._init_settings(
+            USER_DN_TEMPLATE='uid=%(user)s,ou=people,o=test',
+        )
+
+        alice = self.backend.authenticate(username=u'alice', password=u'')
+
+        self.assertEqual(alice, None)
+        self.assertEqual(self.mock_ldap.ldap_methods_called(), [])
+
+    def test_permit_empty_password(self):
+        self._init_settings(
+            USER_DN_TEMPLATE='uid=%(user)s,ou=people,o=test',
+            PERMIT_EMPTY_PASSWORD=True,
+        )
+
+        alice = self.backend.authenticate(username=u'alice', password=u'')
+
+        self.assertEqual(alice, None)
+        self.assertEqual(self.mock_ldap.ldap_methods_called(),
+            ['initialize', 'simple_bind_s'])
+
+    def test_pickle(self):
+        self._init_settings(
+            USER_DN_TEMPLATE='uid=%(user)s,ou=people,o=test',
+            GROUP_SEARCH=LDAPSearch('ou=groups,o=test', self.mock_ldap.SCOPE_SUBTREE),
+            GROUP_TYPE=MemberDNGroupType(member_attr='member'),
+            FIND_GROUP_PERMS=True
+        )
+        self._init_groups()
+        self.mock_ldap.set_return_value('search_s',
+            ("ou=groups,o=test", 2, "(&(objectClass=*)(member=uid=alice,ou=people,o=test))", None, 0),
+            [self.active_gon, self.staff_gon, self.superuser_gon, self.nested_gon]
+        )
+
+        alice0 = self.backend.authenticate(username=u'alice', password=u'password')
+
+        pickled = pickle.dumps(alice0, pickle.HIGHEST_PROTOCOL)
+        alice = pickle.loads(pickled)
+        alice.ldap_user.backend.settings = alice0.ldap_user.backend.settings
+
+        self.assert_(alice is not None)
+        self.assertEqual(self.backend.get_group_permissions(alice), set(["auth.add_user", "auth.change_user"]))
+        self.assertEqual(self.backend.get_all_permissions(alice), set(["auth.add_user", "auth.change_user"]))
+        self.assert_(self.backend.has_perm(alice, "auth.add_user"))
+        self.assert_(self.backend.has_module_perms(alice, "auth"))
+
     def _init_settings(self, **kwargs):
         self.backend.settings = TestSettings(**kwargs)
 
